@@ -6,7 +6,7 @@
 import os
 import numpy as np
 import warnings
-from scipy.io.idl import readsav
+from scipy.io import readsav
 from astropy import constants as const
 
 from .uvdata import UVData
@@ -331,6 +331,8 @@ class FHD(UVData):
         check_extra=True,
         run_check_acceptability=True,
         strict_uvw_antpos_check=False,
+        check_autos=True,
+        fix_autos=True,
     ):
         """
         Read in data from a list of FHD files.
@@ -369,6 +371,12 @@ class FHD(UVData):
         strict_uvw_antpos_check : bool
             Option to raise an error rather than a warning if the check that
             uvws match antenna positions does not pass.
+        check_autos : bool
+            Check whether any auto-correlations have non-zero imaginary values in
+            data_array (which should not mathematically exist). Default is True.
+        fix_autos : bool
+            If auto-correlations with imaginary values are found, fix those values so
+            that they are real-only in data_array. Default is False.
 
         Raises
         ------
@@ -711,4 +719,6 @@ class FHD(UVData):
                 run_check_acceptability=run_check_acceptability,
                 strict_uvw_antpos_check=strict_uvw_antpos_check,
                 allow_flip_conj=True,
+                check_autos=check_autos,
+                fix_autos=fix_autos,
             )
