@@ -192,6 +192,132 @@ def test_read_write_read_carma(tmp_path):
     assert uv_in == uv_out
 
 
+@pytest.mark.filterwarnings("ignore:pamatten in extra_keywords is a list, array")
+@pytest.mark.filterwarnings("ignore:psys in extra_keywords is a list, array or dict")
+@pytest.mark.filterwarnings("ignore:psysattn in extra_keywords is a list, array or")
+@pytest.mark.filterwarnings("ignore:ambpsys in extra_keywords is a list, array or dict")
+@pytest.mark.filterwarnings("ignore:bfmask in extra_keywords is a list, array or dict")
+@pytest.mark.filterwarnings("ignore:LST values stored in this file are not ")
+@pytest.mark.filterwarnings("ignore:The uvw_array does not match the expected values")
+@pytest.mark.filterwarnings("ignore:Writing in the MS file that the units of the data")
+def test_read_carma_miriad_write_ms(tmp_path):
+    """
+    Check a roundtrip between CARMA-MIRIAD and MS formats.
+
+    Note that this check does a few different operations -- these are consolidated into
+    one test because several of them rely upon manipulating the same file object.
+    """
+    pytest.importorskip("casacore")
+    from casacore import tables
+
+    uv_in = UVData()
+    uv_out = UVData()
+    carma_file = os.path.join(DATA_PATH, "carma_miriad")
+    testfile = os.path.join(tmp_path, "outtest_carma_miriad.ms")
+
+    with uvtest.check_warnings(
+        UserWarning,
+        [
+            "Altitude is not present in Miriad file, "
+            "using known location values for SZA.",
+            "The uvw_array does not match the expected values given the antenna "
+            "positions.",
+            "pamatten in extra_keywords is a list, array or dict",
+            "psys in extra_keywords is a list, array or dict",
+            "psysattn in extra_keywords is a list, array or dict",
+            "ambpsys in extra_keywords is a list, array or dict",
+            "bfmask in extra_keywords is a list, array or dict",
+            "Cannot fix the phases of multi phase center datasets, as they were not "
+            "supported when the old phasing method was used, and thus, there "
+            "is no need to correct the data.",
+        ],
+    ):
+        uv_in.read(carma_file, fix_old_proj=True)
+
+    # MIRIAD is missing these in the file, so we'll fill it in here.
+    uv_in.antenna_diameters = np.zeros(uv_in.Nants_telescope)
+    uv_in.antenna_diameters[:6] = 10.0
+    uv_in.antenna_diameters[15:] = 3.5
+
+    # We need to recalculate app coords here for one source ("NOISE"), which was
+    # not actually correctly calculated in the online CARMA system (long story). Since
+    # the MS format requires recalculating apparent coords after read in, we'll
+    # calculate them here just to verify that everything matches.
+    uv_in._set_app_coords_helper()
+    with uvtest.check_warnings(
+        UserWarning,
+        [
+            "The uvw_array does not match the expected values given the antenna ",
+            "pamatten in extra_keywords is a list, array or dict",
+            "psys in extra_keywords is a list, array or dict",
+            "psysattn in extra_keywords is a list, array or dict",
+            "ambpsys in extra_keywords is a list, array or dict",
+            "bfmask in extra_keywords is a list, array or dict",
+            "Writing in the MS file that the units of the data are",
+        ],
+    ):
+        uv_in.write_ms(testfile, clobber=True)
+
+    uv_out.read(testfile)
+
+    # Make sure the MS extra keywords are as expected
+    assert uv_out.extra_keywords["DATA_COL"] == "DATA"
+    assert uv_out.extra_keywords["observer"] == "SZA"
+    uv_in.extra_keywords["DATA_COL"] = "DATA"
+    uv_in.extra_keywords["observer"] = "SZA"
+
+    # make sure filename is what we expect
+    assert uv_in.filename == ["carma_miriad"]
+    assert uv_out.filename == ["outtest_carma_miriad.ms"]
+    uv_in.filename = uv_out.filename
+
+    # Do a quick check on the history to verify they're different.
+    assert uv_in.history != uv_out.history
+    uv_in.history = uv_out.history
+
+    # Final equality check
+    assert uv_in.__eq__(uv_out, allowed_failures=["filename"])
+
+    # Manipulate the table so that all fields have the same name (as is permitted
+    # for MS files), and wipe out the source_ID information
+    tb_field = tables.table(os.path.join(testfile, "FIELD"), ack=False, readonly=False)
+    orig_name = tb_field.getcol("NAME")
+    tb_field.putcol("NAME", ["TEST"] * 3)
+    tb_field.removecols("SOURCE_ID")
+    tb_field.close()
+
+    # Check and see that the naming convention lines up as expected -- only the internal
+    # catalog entries (specifically the names/keys and catalog IDs) should have changed.
+    uv_out.read(testfile)
+    cat_id_dict = {}
+    for idx in range(3):
+        sou_dict = uv_out.phase_center_catalog.pop("TEST-%03i" % idx)
+        sou_dict["cat_id"] = uv_in.phase_center_catalog[orig_name[idx]]["cat_id"]
+        uv_out.phase_center_catalog[orig_name[idx]] = sou_dict
+        cat_id_dict[idx] = uv_in.phase_center_catalog[orig_name[idx]]["cat_id"]
+
+    uv_out.phase_center_id_array = np.array(
+        [cat_id_dict[key] for key in uv_out.phase_center_id_array], dtype=int
+    )
+
+    # Final equality check
+    assert uv_in.__eq__(uv_out, allowed_failures=["filename"])
+
+    # Last but not least, change the coord system so that the catalog frames are the
+    # same as phase_center_frame
+    uv_in.phase_center_frame = "fk5"
+    uv_in._set_app_coords_helper()
+
+    # Check to make sure we raise an error if overwritting a file w/o clobber enabled
+    with pytest.raises(OSError, match="File exists; skipping"):
+        uv_in.write_ms(testfile, clobber=False)
+
+    # Do one last read/write/check for equality
+    uv_in.write_ms(testfile, clobber=True)
+    uv_out.read(testfile)
+    assert uv_in.__eq__(uv_out, allowed_failures=["filename"])
+
+
 @pytest.mark.filterwarnings("ignore:LST values stored in this file are not ")
 @pytest.mark.filterwarnings("ignore:The uvw_array does not match the expected values")
 def test_read_miriad_write_uvfits(uv_in_uvfits):
@@ -904,6 +1030,7 @@ def test_miriad_antenna_diameters(uv_in_paper):
     assert uv_in == uv_out
 
 
+@pytest.mark.filterwarnings("ignore:Fixing auto-correlations to be be real-only,")
 @pytest.mark.filterwarnings("ignore:The uvw_array does not match the expected values")
 def test_miriad_write_read_diameters(tmp_path):
     uv_in = UVData()
@@ -1373,6 +1500,9 @@ def test_read_ms_write_miriad_casa_history(tmp_path):
     assert ms_uv.filename == ["day2_TDEM0003_10s_norx_1src_1spw.ms"]
     miriad_uv.filename = ms_uv.filename
 
+    # propagate scan numbers to the uvfits, ONLY for comparison
+    miriad_uv.scan_number_array = ms_uv.scan_number_array
+
     assert miriad_uv == ms_uv
 
 
@@ -1531,6 +1661,7 @@ def test_antpos_units(casa_uvfits, tmp_path):
     assert np.allclose(aantpos, uv.antenna_positions)
 
 
+@pytest.mark.filterwarnings("ignore:Fixing auto-correlations to be be real-only,")
 @pytest.mark.filterwarnings("ignore:The uvw_array does not match the expected values")
 def test_readmiriad_write_miriad_check_time_format(tmp_path):
     """
